#include <iostream>
#include <vector>
#include <set>
#include <numeric>
#include <iomanip>
#include <random>

#define int long long

using namespace std;

random_device rd;
mt19937_64 get_rand(rd());

bool adj[30][30];

const int INF = 1e9;
const int MOD1 = get_rand() % INF + INF;
const int MOD2 = get_rand() % INF + INF;

struct Hash {
	int v1 = 0, v2 = 0;
	Hash(int v1, int v2) : v1(v1% MOD1), v2(v2% MOD2) {}
	Hash(int v) : v1(v% MOD1), v2(v% MOD2) {}
	Hash() {}
};

const Hash B(37);

Hash operator+(Hash a, Hash b) { return {(a.v1 + b.v1) % MOD1, (a.v2 + b.v2) % MOD2}; }
Hash operator*(Hash a, Hash b) { return {(a.v1 * b.v1) % MOD1, (a.v2 * b.v2) % MOD2}; }
Hash operator-(Hash a, Hash b) { return {(a.v1 - b.v1 + MOD1) % MOD1, (a.v2 - b.v2 + MOD2) % MOD2}; }
bool operator==(Hash a, Hash b) { return a.v1 == b.v1 && a.v2 == b.v2; }

Hash e[500010];

void init(vector <int>& s, vector <Hash>& h) {
	e[0] = 1;
	for (int i = 1; i < 500010; i++)
		e[i] = e[i - 1] * B;
	h.resize(s.size());
	h[0] = Hash(s[0]);
	for (int i = 1; i < s.size(); i++)
		h[i] = h[i - 1] * B + Hash(s[i]);
}

Hash get_hash(int l, int r, vector <Hash>& h) {
	return h[r] - (l ? h[l - 1] : Hash(0)) * e[r - l + 1];
}

signed main()
{
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout << fixed << setprecision(10);

	int k, n;
	cin >> k >> n;
	for (int i = 0; i < k; i++)
		for (int j = 0; j < k; j++) {
			char c;
			cin >> c;
			adj[i][j] = c - '0';
		}
	vector <int> a(n);
	for (int i = 0; i < n; i++) {
		char c;
		cin >> c;
		a[i] = c - 'a' + 2;
	}

	vector <int> dp(n);
	iota(dp.begin(), dp.end(), 1);
	for (int i = n - 2; i >= 0; i--) {
		int j = i + 1;
		while (j < n && adj[a[i] - 2][a[j] - 2]) 
			j = dp[j];
		dp[i] = j;
	}

	vector <int> b = a;
	b.emplace_back(1);
	for (int i : a)
		b.emplace_back(i);
	vector <Hash> h;
	init(b, h);

	auto suf_less = [&b, &n, &h](int i, int j) {
		int l = 0, r = n;
		while (l + 1 < r) {
			int m = l + r >> 1;

			if (get_hash(i, i + m - 1, h) == get_hash(j, j + m - 1, h))
				l = m;
			else
				r = m;
		}

		return b[i + r - 1] < b[j + r - 1];
	};

	if (k <= 2)
		for (int i = 0; i < n; i = dp[i]) {
			int mn = i + 1;
			for (int j = i + 1; j <= dp[i]; j++)
				if (j == n || suf_less(j, mn))
					mn = j;

			cout << (char)(a[i] - 2 + 'a');
			for (int j = mn; j < dp[i]; j++)
				cout << (char)(a[j] - 2 + 'a');
		}
	else
		for (int i = 0; i < n; ) {
			cout << (char)(a[i] - 2 + 'a');

			int mn = i + 1;
			for (int j = i + 1; j <= dp[i]; j++)
				if (j == n || suf_less(j, mn))
					mn = j;

			i = mn;
		}
	return 0;
}
