#include <iostream>
#include <algorithm>
#include <vector>
#include <set>
#include <cassert>

using namespace std;

const int N = 3e5 + 7;
const int INF = 1e9 + 239;

struct Edge {
	int u, v, c;

	Edge() {}

	Edge(int u_, int v_, int c_) {
		u = u_;
		v = v_;
		c = c_;
	}

	bool operator<(const Edge &other) const {
		return c < other.c;
	}
};

namespace DSU {
	int p[N];
	int rank[N];

	int get(int v) {
		if (p[v] != v) {
			p[v] = get(p[v]);
		}
		return p[v];
	}

	int merge(int a, int b) {
		a = get(a);
		b = get(b);
		if (a == b) {
			return 0;
		} else {
			if (rank[a] < rank[b]) swap(a, b);
			p[b] = a;
			if (rank[a] == rank[b]) rank[a]++;
			return 1;
		}
	}

	void init(int n) {
		for (int i = 0; i < n; i++) {
			p[i] = i;
			rank[i] = 0;
		}
	}
}

int n;
vector<Edge> E;

int ans[N];

int used[N];
vector<pair<int, int>> g[N];

void dfs(int v, int cmax, int cmin) {
	used[v] = 1;
	ans[v] = min(ans[v], cmax + cmin);
	for (auto [u, c] : g[v]) {
		if (!used[u]) {
			dfs(u, max(cmax, c), min(cmin, c));
		}
	}
}

void relax(int x) {
	for (int i = 0; i < n; i++) {
		g[i].clear();
	}
	DSU::init(n);
	for (auto e : E) {
		if (e.c >= x) {
			if (DSU::merge(e.u, e.v)) {
				g[e.u].push_back({e.v, e.c});
				g[e.v].push_back({e.u, e.c});
			}
		}
	}
	fill(used, used + n, 0);
	dfs(0, -INF, INF);
}

signed main() {

	int m;
	cin >> n >> m;
	fill(ans, ans + n, INF);
	for (int i = 0; i < m; i++) {
		int u, v, c;
		cin >> u >> v >> c;
		u--;
		v--;
		E.push_back(Edge(u, v, c));
	}
	sort(E.begin(), E.end());
	for (auto t : E) {
		relax(t.c);
	}
	for (int i = 1; i < n; i++) {
		cout << ans[i] << '\n';
	}
}