#include <bits/stdc++.h>

using namespace std;

typedef long long ll;

const ll INF = 1e9;
const ll INFLL = 1e18;

struct Edge{
    ll to, tm, mark;
    ll is_bridge;
};

void dfs(int v, int eid, int &T, vector<vector<Edge> > &g, vector<ll> &in, vector<ll> &out, vector<ll> &up)
{
    in[v] = up[v] = T++;
    for (auto e : g[v])
    {
        int u = e.to;
        if (in[u] == 0)
        {
            dfs(u, e.mark, T, g, in, out, up);
            up[v] = min(up[v], up[u]);
        }
        else if (e.mark != eid)
        {
            up[v] = min(up[v], in[u]);
        }
    }
    out[v] = T++;
}

void dfs(int v, int cl, vector<int> &color, vector<vector<Edge> > &g)
{
    color[v] = cl;
    for (auto e : g[v])
    {
        if (e.is_bridge == 1) continue;
        int u = e.to;
        if (color[u] == -1) dfs(u, cl, color, g);
    }
}

void dfs2(int v, int p, vector<int> &vis2, vector<ll> &ans, vector<vector<pair<ll, ll> > > &g)
{
    vis2[v] = 1;
    for (auto e : g[v]) if (e.first != p)
    {
        int u = e.first;
        ans[u] = min(ans[u], min(ans[v], e.second));
        dfs2(u, v, vis2, ans, g);
    }
}

vector<ll> solve_fixed_mx(int n, int m, vector<int> fr, vector<int> to, vector<int> tm, int lim)
{
    vector<vector<Edge> > g(n);
    for (int i = 0; i < m; i++)
    {
        if (tm[i] <= lim)
        {
            g[fr[i]].push_back({to[i], tm[i], i, 0});
            g[to[i]].push_back({fr[i], tm[i], i, 0});
        }
    }

    vector<ll> in(n), out(n), up(n);
    int T = 1;
    dfs(0, -1, T, g, in, out, up);

    for (int i = 0; i < n; i++)
    {
        for (int j = 0; j < g[i].size(); j++)
        {
            Edge e = g[i][j];
            int v = i, u = e.to;
            if (in[v] <= in[u] && out[u] <= out[v] && up[u] == in[u]) g[i][j].is_bridge = 1;
            if (in[u] <= in[v] && out[v] <= out[u] && up[v] == in[v]) g[i][j].is_bridge = 1;
        }
    }

    vector<int> color(n, -1);
    ll cl = 0;
    for (int i = 0; i < n; i++)
    {
        if (color[i] == -1)
        {
            dfs(i, cl, color, g);
            cl++;
        }
    }

    vector<vector<pair<ll, ll> > > comp_g(cl);
    vector<ll> ans(cl, INFLL);
    for (int i = 0; i < n; i++)
    {
        for (int j = 0; j < g[i].size(); j++)
        {
            Edge e = g[i][j];
            int v = i, u = e.to;
            if (color[v] == color[u]) ans[color[v]] = min(ans[color[v]], e.tm);
            else
            {
                comp_g[color[v]].push_back({color[u], e.tm});
            }
        }
    }

    vector<int> vis2(cl);
    dfs2(color[0], color[0], vis2, ans, comp_g);

    vector<ll> res(n);
    for (int i = 1; i < n; i++)
    {
        res[i] = ans[color[i]] + lim;
    }
    return res;
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(0);

    int n, m;
    cin >> n >> m;
    vector<int> fr(m), to(m), tm(m);
    int C = 0;
    for (int i = 0; i < m; i++)
    {
        cin >> fr[i] >> to[i] >> tm[i];
        fr[i]--, to[i]--;
        if (abs(fr[i] - to[i]) == 1)
            C = max(C, tm[i]);
    }
    vector<ll> ans(n, INFLL);
    vector<ll> res = solve_fixed_mx(n, m, fr, to, tm, C);
    for (int i = 0; i < n; i++) ans[i] = min(ans[i], res[i]);
    for (int i = 1; i < n; i++)
        cout << ans[i] << "\n";
}



/*
6 7
1 2 1000000000
2 3 2
3 4 3
4 1 1000000000
1 5 1000000000
5 6 1
5 6 2
*/
