#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <cstring>
#include <set>
#include <map>
#include <queue>
#include <random>
#include <iomanip>

using namespace std;

template<typename T> void uin(T &a, T b) {
	if (b < a) a = b;
}
template<typename T> void uax(T &a, T b) {
	if (b > a) a = b;
}

#define left left228
#define right right228
#define all(v) v.begin(), v.end()

const int N = 150 * 1000 + 7;

int n, m;
vector<int> g[N];
map< pair<int, int>, int > edge_bal;
int bal[N];
bool fall[N];
bool used[N];
bool canMatch = true;

vector< pair< pair<int, int>, pair<int, int> > > ans;

void dfs(int v, int par = -1) {
	used[v] = true;
	vector<int> sons;
	int children = 0;
	for (int to : g[v]) {
		if (to != par) {
			dfs(to, v);
			++children;
			if (!fall[to]) {
				sons.push_back(to);
			}
		}
	}
	// cout << "v=" << v << " sons : ";
	// for (auto p : sons) {
	// 	cout << "(" << p << ", bal=" << bal[p] << ") ";
	// }
	// cout << endl;
	if (children == 0) {
		bal[v] = 0;
		bal[v] = edge_bal[{v, par}];
		return ;
	}
	bool is_up = true;
	bool is_down = true;
	int sum_bal = 0;
	for (int son : sons) {
		sum_bal += bal[son];
	}

	if (abs(sum_bal) > 1) {
		canMatch = false;
		return ;
	}
	if (par == -1 && sum_bal != 0) {
		// root of tree
		canMatch = false;
		return ;
	}
	vector< pair<int, int> > up_edges, down_edges;
	for (int son : sons) {
		if (bal[son] == 0) {
			up_edges.push_back({son, v});
			down_edges.push_back({v, son});
		} else if (bal[son] == 1) {
			up_edges.push_back({son, v});
		} else {
			down_edges.push_back({v, son});
		}
	}
	while (!up_edges.empty() && !down_edges.empty()) {
		pair<int, int> e1 = up_edges.back();
		pair<int, int> e2 = down_edges.back();
		ans.emplace_back(e1, e2);
		up_edges.pop_back();
		down_edges.pop_back();
	}
	bool was_deleted = false;
	if (!up_edges.empty()) {
		pair<int, int> e = up_edges.back();
		if (edge_bal[{v, par}] >= 0) {
			ans.emplace_back(e, make_pair(v, par));
			edge_bal[{v, par}] -= 1;
			edge_bal[{par, v}] += 1;

			was_deleted = true;
		} else {
			canMatch = false;
			return;
		}
	}
	if (!down_edges.empty()) {
		pair<int, int> e = down_edges.back();
		if (edge_bal[{par, v}] >= 0) {
			ans.emplace_back(make_pair(par, v), e);
			--edge_bal[{par, v}];
			++edge_bal[{v, par}];

			was_deleted = true;
		} else {
			canMatch = false;
			return ;
		}
	}
	bal[v] = edge_bal[{v, par}];
	if (was_deleted && edge_bal[{v, par}] == 0) {
		fall[v] = true;
	}
}

void solve() {
	for (int v = 1; v <= n; ++v) {
		if (!used[v]) {
			dfs(v);
		}
	}
	if (!canMatch) {
		cout << "No\n";
		exit(0);
	}
	cout << "Yes\n";
	for (auto P : ans) {
		cout << P.first.first << ' ' << P.first.second << ' ' << P.second.first << ' ' << P.second.second << '\n';
	}
}

signed main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cin >> n >> m;
	for (int i = 0; i < m; ++i) {
		int u, v;
		cin >> u >> v;

		edge_bal[{u, v}] += 1;
		edge_bal[{v, u}] -= 1;

		g[u].push_back(v);
		g[v].push_back(u);
	}
	{
		for (int v = 1; v <= n; ++v) {
			sort(all(g[v]));
			g[v].erase(unique(all(g[v])), g[v].end());
		}
	}
	solve();
	return 0;
}


/*

5 6
1 2
2 1
1 5
2 3
2 4
4 2



4 4
2 1
2 3
2 4
4 2


4 4
1 2
2 1
3 4
4 3

*/