#include <iostream>
#include <vector>
#include <sstream>
#include <set>
#include <map>
#include <assert.h>
#include <cmath>
#include <algorithm>

using namespace std;
typedef long long LL;
typedef unsigned long long ULL;

struct Graph {
	int size;
	vector<vector<int>> neighbours;
	map<pair<int, int>, int> costs;
	vector<int> minOnPath;
	vector<int> maxOnPath;

	Graph(){}

	Graph(int s) {
		size = s;
		neighbours.resize(s);
		minOnPath.resize(s);
		maxOnPath.resize(s);
	}
	
	void addEdge(int a, int b, int cost) {
		neighbours[a].push_back(b);
		neighbours[b].push_back(a);
		costs[{a, b}] = cost;
		costs[{b, a}] = cost;
	}

	void dfs(int v, int p = -1) {
		
		if (p == 0) {
			minOnPath[v] = costs[{v, p}];
			maxOnPath[v] = costs[{v, p}];
		}
		else {
			minOnPath[v] = p != -1 ? min(costs[{v, p}], minOnPath[p]) : 0;
			maxOnPath[v] = p != -1 ? max(maxOnPath[p], costs[{v, p}]) : 0;
		}
		
		for (auto u : neighbours[v])
		{
			if (u != p) {
				dfs(u, v);
			}
		}
	}
};

int main() {
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	int n, m;
	cin >> n >> m;

	Graph G(n);

	for (int i = 0; i < m; i++)
	{
		int a, b, cost;
		cin >> a >> b >> cost;
		a--;
		b--;
		G.addEdge(a, b, cost);
	}

	G.dfs(0);

	for (int i = 1; i < n; i++)
	{
		cout << G.maxOnPath[i] + G.minOnPath[i] << endl;
	}

}