#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

vector<vector<pair<int, int>>> adj;
vector<int> min_answer, max_answer;


void dfs(int v, int min_ans = 1000000000, int max_ans = -1, int pre = -1) {
    min_answer[v] = min_ans;
    max_answer[v] = max_ans;
    for (auto p: adj[v]) {
        if (p.first != pre) dfs(p.first, min(min_ans, p.second), max(max_ans, p.second), v);
    }
}

struct EDGE {
    int from, to, weight;

    EDGE (int f, int t, int w) {
        from = f; to = t; weight = w;
    }
};


bool cmp(EDGE a, EDGE b) {
    return a.weight < b.weight;
}

vector<EDGE> edges;

struct DSU {
    vector<int> pre, size;

    DSU(int n) {
        pre.resize(n + 1);
        size.resize(n + 1);
        for (int i = 1; i <= n; i++) {
            pre[i] = i;
            size[i] = 1;
        }
    }

    int get_pre(int a) {
        if (pre[a] == a) return a;
        return pre[a] = get_pre(pre[a]);
    }

    void merge(int a, int b) {
        a = get_pre(a);
        b = get_pre(b);
        if (size[a] < size[b]) swap(a, b);
        pre[b] = a;
        size[a] += size[b];
    }

    bool in_one(int a, int b) {
        return get_pre(a) == get_pre(b);
    }
};

int main() {
    int n, m, a, b, t;
    cin >> n >> m;

    min_answer.resize(n + 1);
    max_answer.resize(n + 1);
    adj.resize(n + 1);

    if (m == n - 1) {
        for (int i = 0; i < m; i++) {
            cin >> a >> b >> t;
            adj[a].emplace_back(b, t);
            adj[b].emplace_back(a, t);
        }
        dfs(1);
        for (int k = 2; k <= n; k++) {
            cout << max_answer[k] + min_answer[k] << endl;
        }
    } else {
        DSU d(n);
        for (int i = 0; i < m; i++) {
            cin >> a >> b >> t;
            edges.emplace_back(a, b, t);
        }

        sort(edges.begin(), edges.end(), &cmp);
        for (auto e: edges) {
            if (!d.in_one(e.from, e.to)) {
                adj[e.to].emplace_back(e.from, e.weight);
                adj[e.from].emplace_back(e.to, e.weight);
                d.merge(e.from, e.to);
            }
        }

        dfs(1);
        for (int k = 2; k <= n; k++) {
            cout << max_answer[k] + min_answer[k] << endl;
        }
    }
}