#include <algorithm>
#include <cassert>
#include <iostream>
#include <vector>

using namespace std;

#define SZ(c) (int)c.size()

// #define int long long

const int N = 5e5 + 7;

int n, nq;
int x[N], h[N];
pair<pair<int, int>, int> q[N];
int lx[N], rx[N];

bool used[N];

bool ok_left(int i) {
  if (x[i] - h[i] < x[0]) {
    return false;
  }
  for (int j = 0; j < i; ++j) {
    if (!used[j] && x[j] > x[i] - h[i]) {
      return false;
    }
  }
  return true;
}

bool ok_right(int i) {
  if (x[i] + h[i] > x[n - 1]) {
    return false;
  }
  for (int j = i + 1; j < n; ++j) {
    if (!used[j] && x[j] < x[i] + h[i]) {
      return false;
    }
  }
  return true;
}

int stupid_get(int l, int r) {
  fill(used, used + n, false);
  int res = 0;
  while (true) {
    bool flag = false;
    for (int i = l; i <= r; ++i) {
      if (!used[i] && (ok_left(i) || ok_right(i))) {
        used[i] = true;
        flag = true;
        ++res;
      }
    }
    if (!flag) {
      break;
    }
  }
  return res;
}

int get(int l, int r) {
  int clx = (l == 0 ? x[l] : x[l - 1]), crx = (r == n - 1 ? x[r] : x[r + 1]);
  int res = 0;
  for (int j = l; j <= r; ++j) {
    res += (lx[j] >= clx || rx[j] <= crx);
  }
  return res;
}

void build() {
  vector<int> stack;
  for (int i = 0; i < n; ++i) {
    lx[i] = x[i] - h[i];
    while (!stack.empty() && x[stack.back()] > lx[i]) {
      if (x[stack.back()] + h[stack.back()] > x[i]) {
        lx[i] = min(lx[i], x[stack.back()] - h[stack.back()]);
      }
      stack.pop_back();
    }
  }
  for (int i = 0; i < n; ++i) {
    rx[i] = x[i] + h[i];
    int j = i + 1;
    vector<int> stack = {i};
    while (j < n && x[j] < rx[i]) {
      while (x[stack.back()] + h[stack.back()] <= x[j]) {
        stack.pop_back();
      }
      if (x[j] - h[j] < x[stack.back()]) {
        rx[i] = max(rx[i], x[j] + h[j]);
        stack.push_back(j);
      }
      ++j;
    }
  }
}

void scan() {
  cin >> n >> nq;
  for (int i = 0; i < n; ++i) {
    cin >> x[i] >> h[i];
  }
  for (int i = 0; i < nq; ++i) {
    int l, r;
    cin >> l >> r;
    --l, --r;
    q[i] = {{l, r}, i};
  }
}

void print_test() {
  cout << n << ' ' << nq << '\n';
  for (int i = 0; i < n; ++i) {
    cout << x[i] << ' ' << h[i] << '\n';
  }
}

void gen_test() {
  const int H = 10;
  n = 30;
  x[0] = 1;
  h[0] = rand() % H + 1;
  for (int i = 1; i < n; ++i) {
    x[i] = rand() % (H + 2) + x[i - 1];
    h[i] = rand() % H + 1;
  }
  nq = 20;
  for (int i = 0; i < nq; ++i) {
    int r = rand() % n;
    int l = rand() % (r + 1);
    q[i] = {{l, r}, i};
  }
}

void stresstest() {
  int test_n = 1;
  while (test_n++) {
    cout << "TEST #" << test_n++ << '\n';
    gen_test();
    build();
    for (int i = 0; i < nq; ++i) {
      int l = q[i].first.first, r = q[i].first.second;
      if (get(l, r) != stupid_get(l, r)) {
        cout << "FAIL\n";
        print_test();
        cout << l + 1 << ' ' << r + 1 << '\n';
        cout << "MY ANS: " << get(l, r) << '\n';
        cout << "R8 ANS: " << stupid_get(l, r) << '\n';
        system("pause");
      }
    }
  }
}

signed main() {
  ios_base::sync_with_stdio(false);
  cin.tie(NULL);
  cout.tie(NULL);
  // stresstest();
  scan();
  build();
  for (int i = 0; i < nq; ++i) {
    cout << get(q[i].first.first, q[i].first.second) << '\n';
  }
  // system("pause");
  return 0;
}
/*
4 1
1 10
3 7
10 8
14 1
1 4
*/